# coding:utf-8
import datetime as dt
from typing import Callable
from functools import wraps
from bs4 import BeautifulSoup

from src.browser import Browser
from src.utils.color import Color


class AttendanceManager(object):
    def __init__(self, headless=True):
        self.headless = headless
        self.browser = Browser(headless=headless)
        self.driver = self.browser.driver

    def confirm(self, method):
        if method in dir(self):
            if (fun := getattr(self, method)):
                fun()
            self.exit()

    def open_attendance(self):
        return self.browser.open_attendance()

    def open_prev_attendance(self):
        return self.browser.open_prev_attendance()

    def current_date(self):
        return dt.datetime.now().day

    def history(self):
        """
        dakoker history実行時に走る
        """
        if self.open_attendance():
            times = self.get_timetable(self.get_attendance)
            self.print_timetable(times)

    def overtime(self):
        """
        dakoker overtime 実行時に走る
        """
        if self.open_attendance():
            overtimes = self.get_timetable(self.get_overtime)
            self.print_overtime(overtimes)

    def prev_overtime(self):
        """
        dakoker prev_overtime 実行時に走る
        """
        if self.open_prev_attendance():
            overtimes = self.get_timetable(self.get_overtime)
            self.print_overtime(overtimes)

    def get_timetable(self, getter: Callable) -> list:
        """
        getterで指定した方法でタイムテーブルを取得する
        """
        timetable = getter()
        for i, time in enumerate(timetable):
            timetable[i] = [time for time in time.strings]

        return timetable

    def get_attendance(self) -> list:
        html = self.driver.page_source.encode("utf-8")
        soup = BeautifulSoup(html, "html.parser")
        attendances = soup.find_all("td", class_="column-attendance")

        attendance_array = []
        time = 0
        while time < len(attendances):
            attendance_array.append(attendances[time : time + 4])
            time += 4

        return attendance_array[self.current_date() - 1]

    def get_overtime(self) -> list:
        html = self.driver.page_source.encode("utf-8")
        soup = BeautifulSoup(html, "html.parser")
        normal = (
            soup.select(".attendance-table-contents")[3].select("tr")[2].select("td")[3]
        )
        designated = (
            soup.select(".attendance-table-contents")[3].select("tr")[3].select("td")[3]
        )
        law = (
            soup.select(".attendance-table-contents")[3].select("tr")[4].select("td")[3]
        )

        return [normal, designated, law]

    def exit(self):
        self.driver.close()

    def printer(func: Callable) -> Callable:
        """
        print系メソッド用のラッパー関数
        """

        @wraps(func)
        def newfunc(*args) -> None:
            print("================================")
            func(*args)
            print("================================")

        return newfunc

    @printer
    def print_timetable(self, times):
        texts = [
            Color.get_colored(Color.BOLD, "出勤:     ") + ", ".join(times[0]),
            Color.get_colored(Color.BOLD, "退勤:     ") + ", ".join(times[1]),
            Color.get_colored(Color.BOLD, "休憩開始: ") + ", ".join(times[2]),
            Color.get_colored(Color.BOLD, "休憩終了: ") + ", ".join(times[3]),
        ]
        for text in texts:
            print(text)

    @printer
    def print_overtime(self, times):
        texts = [
            "各残業時間",
            Color.get_colored(Color.BOLD, "平日:     ") + ", ".join(times[0]),
            Color.get_colored(Color.BOLD, "所定休日: ") + ", ".join(times[1]),
            Color.get_colored(Color.BOLD, "法定休日: ") + ", ".join(times[2]),
        ]
        for text in texts:
            print(text)



# coding:utf-8
from src.browser import Browser
from src.utils.color import Color
from src.utils.calc import Calc


class StampManager(object):
    def __init__(self):
        self.browser = Browser()
        self.driver = self.browser.driver

    def stamp(self, method):
        if self.login() and method in dir(self):
            if (fun := getattr(self, method)):
                fun()
            self.exit()

    def login(self):
        return self.browser.login()

    def clock_execute(self, selector, time_prefix):
        if self.driver.current_url != self.browser.MYPAGE_URL:
            print("Please login.")

        self.driver.find_element_by_class_name(selector).click()
        print(time_prefix + Calc.current_time())

    def start(self):
        selector = "attendance-card-time-stamp-clock-in"
        prefix = Color.get_colored(Color.BOLD, "出勤: ")
        self.clock_execute(selector, prefix)

        Color.print(Color.GREEN, "打刻が完了しました。良い一日を！")

    def end(self):
        selector = "attendance-card-time-stamp-clock-out"
        prefix = Color.get_colored(Color.BOLD, "退勤: ")
        self.clock_execute(selector, prefix)

        Color.print(Color.GREEN, "打刻が完了しました。一日お疲れ様でした！")

    def start_break(self):
        selector = "attendance-card-time-stamp-start-break"
        prefix = Color.get_colored(Color.BOLD, "休憩開始: ")
        self.clock_execute(selector, prefix)

        Color.print(Color.GREEN, "休憩開始の打刻が完了しました。")

    def end_break(self):
        selector = "attendance-card-time-stamp-end-break"
        prefix = Color.get_colored(Color.BOLD, "休憩終了: ")
        self.clock_execute(selector, prefix)

        Color.print(Color.GREEN, "休憩終了の打刻が完了しました。")

    def exit(self):
        self.driver.close()



# coding:utf-8
import os
import pickle
import getpass
import keyring
from halo import Halo


class UserInfoManager(object):
    CORP_ID = "CORP_ID"
    USER_ID = "USER_ID"
    USER_PASS = "USER_PASS"
    MF_SERVICE = "MF_SERVICE"
    USER_INFO_PATH = os.environ["HOME"] + "/.local/share/dakoker"

    def get(self) -> hash:
        user_info = self.get_cached()
        if not user_info:
            user_info = {}
            print("Please enter your login info.")
            user_info[self.CORP_ID] = input("company ID: ")
            user_info[self.USER_ID] = input("user ID or email address: ")
            user_info[self.USER_PASS] = getpass.getpass("password: ")

        return user_info

    def get_cached(self):
        if os.path.isfile(self.USER_INFO_PATH + "/user_info.pkl"):
            with open(self.USER_INFO_PATH + "/user_info.pkl", "rb") as f:
                info = pickle.load(f)
                passwd = keyring.get_password(self.MF_SERVICE, info[self.USER_ID])
                if passwd:
                    info[self.USER_PASS] = passwd
                    return info

        return None

    def save(self, user_info):
        if not os.path.isdir(self.USER_INFO_PATH):
            os.makedirs(self.USER_INFO_PATH)

        keyring.set_password(
            self.MF_SERVICE, user_info[self.USER_ID], user_info[self.USER_PASS]
        )

        del user_info[self.USER_PASS]
        pickle.dump(user_info, open(self.USER_INFO_PATH + "/user_info.pkl", "wb"))

    @classmethod
    def remove(cls) -> bool:
        if os.path.isfile(cls.USER_INFO_PATH + "/user_info.pkl"):
            os.remove(cls.USER_INFO_PATH + "/user_info.pkl")
            return True

        return False

    @classmethod
    def remove_with_message(cls):
        spinner = Halo(text="Remove your local data...", spinner="dots")
        if UserInfoManager.remove():
            spinner.succeed("Data Successfully deleted.")
        else:
            spinner.warn("Data not found.")



# coding:utf-8


class Color(object):
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    PURPLE = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"
    BOLD = "\033[1m"
    END = "\033[0m"

    @classmethod
    def get_colored(cls, color, text):
        return color + text + cls.END

    @classmethod
    def print(cls, color, text):
        print(cls.get_colored(color, text))



# coding:utf-8
from pick import pick
from halo import Halo
from time import sleep
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException

from src.utils.color import Color
from src.user_info_manager import UserInfoManager


class Browser(object):
    TIMEOUT = 3
    ROOT_URL = "https://attendance.moneyforward.com"
    LOGIN_URL = ROOT_URL + "/employee_session/new"
    MYPAGE_URL = ROOT_URL + "/my_page"
    ATTENDANCE_URL = MYPAGE_URL + "/attendances"
    LOGIN_SUCCEED = "ログインに成功しました。"
    LOGIN_FAILED = "ログインに失敗しました。"

    DRIVER = "DRIVER"
    SAFARI = "Safari Drievr"
    CHROME = "Chrome Drievr"

    def __init__(self, headless=True):
        self.headless = headless
        self.info_manager = UserInfoManager()
        self.setup_userinfo()

    def setup_userinfo(self):
        self.userinfo = self.info_manager.get()
        self.set_driver_to_userinfo()
        self.set_driver()

    def set_driver(self):
        if self.userinfo[self.DRIVER] == self.SAFARI:
            self.driver = webdriver.Safari()
        else:
            options = webdriver.ChromeOptions()
            options.headless = self.headless
            self.driver = webdriver.Chrome(chrome_options=options)

    def set_driver_to_userinfo(self):
        if self.DRIVER not in self.userinfo.keys():
            message = "ブラウザドライバを選択してください:"
            options = [self.SAFARI, self.CHROME]
            option, _ = pick(options, message)
            self.userinfo[self.DRIVER] = option

    def login(self):
        spinner = Halo(text="ログインページをロードしています...", spinner="dots")
        spinner.start()
        self.driver.get(self.LOGIN_URL)
        spinner.succeed("ログインページのロードを完了しました。")

        spinner = Halo(text="ログイン中...", spinner="dots")
        spinner.start()
        return self.login_with_userinfo(spinner)

    def login_with_userinfo(self, spinner):
        self.driver.find_element_by_id(
            "employee_session_form_office_account_name"
        ).send_keys(self.userinfo[self.info_manager.CORP_ID])
        self.driver.find_element_by_id(
            "employee_session_form_account_name_or_email"
        ).send_keys(self.userinfo[self.info_manager.USER_ID])
        self.driver.find_element_by_id("employee_session_form_password").send_keys(
            self.userinfo[self.info_manager.USER_PASS]
        )

        self.driver.find_element_by_class_name(
            "attendance-before-login-card-button"
        ).click()

        return self.check_login(spinner)

    def check_login(self, spinner):
        try:
            WebDriverWait(self.driver, self.TIMEOUT).until(
                EC.presence_of_element_located((By.CLASS_NAME, "attendance-card-title"))
            )
            self.info_manager.save(self.userinfo)
            spinner.succeed(self.LOGIN_SUCCEED)
            return True

        except TimeoutException:
            if self.driver.find_elements(By.CLASS_NAME, "is-error") != 0:
                Color.print(Color.RED, "\n企業ID, ユーザーID もしくはパスワードが間違っています。")
                spinner.fail(self.LOGIN_FAILED)
                UserInfoManager.remove()
                return False
            else:
                Color.print(Color.RED, "\nログイン中にタイムアウトしました。")
                spinner.fail(self.LOGIN_FAILED)
                return False

    def open_attendance(self):
        if self.login():
            spinner = Halo(text="日次勤怠ページをロードしています...", spinner="dots")
            self.driver.get(self.ATTENDANCE_URL)

        try:
            WebDriverWait(self.driver, self.TIMEOUT).until(
                EC.presence_of_element_located(
                    (By.CLASS_NAME, "modal-controller-my-page-attendances")
                )
            )
            spinner.succeed("日次勤怠ページのロードを完了しました。")
            return True

        except TimeoutException:
            return False

    def open_prev_attendance(self):
        if self.open_attendance():
            spinner = Halo(text="先月の日次勤怠ページをロードしています...", spinner="dots")
            self.driver.find_element_by_class_name(
                "attendance-table-header-month-range-previous"
            ).click()
            # 先月の日次勤怠ページ表示までsleepを挟む
            sleep(1)

        try:
            WebDriverWait(self.driver, self.TIMEOUT).until(
                EC.presence_of_element_located(
                    (By.CLASS_NAME, "modal-controller-my-page-attendances")
                )
            )
            spinner.succeed("先月の日次勤怠ページのロードを完了しました。")
            return True

        except TimeoutException:
            return False
